<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Product_Search_Filters
 * @subpackage Psf_Product_Search_Filters/admin
 * @author     KanKode <hello@kankode.com>
 */
class Psf_Product_Search_Filters_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of this plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {
		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/styles.css', [], $this->version, 'all' );
		wp_register_style( 'select2', plugins_url( 'css/select2.min.css', __FILE__ ), [], $this->version, 'all' );
		wp_enqueue_style( 'select2' );
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		wp_register_script( 'select2', plugin_dir_url( __FILE__ ) . 'js/select2.min.js', ['jquery'], null, true );
		wp_enqueue_script( 'select2' );

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/scripts.js', [ 'jquery' ], $this->version, true );
		wp_localize_script( $this->plugin_name, 'wpsObject', [
			'placeholderSearchProduct'      => esc_attr__( 'Search for a product...', 'product-search-and-filters' ),
			'placeholderSearchCategory'     => esc_attr__( 'Search for a category...', 'product-search-and-filters' ),
			'placeholderSearchCustomFields' => esc_attr__( 'Search for a custom attribute...', 'product-search-and-filters' ),
			'placeholderSearchAttribute'    => esc_attr__( 'Search for a attribute...', 'product-search-and-filters' ),
			'ajaxUrl'                       => esc_url( admin_url( 'admin-ajax.php' ) ),
		] );
	}

	/**
	 * Add settings page
	 */
	public function add_settings_pages() {
		add_submenu_page( 'woocommerce', esc_html__( 'Search and Filters', 'product-search-and-filters' ), esc_html__( 'Search and Filters', 'product-search-and-filters' ), 'manage_options', 'product-search-and-filters-settings', [
			$this,
			'render_plugin_settings_page',
		], 80 );
	}

	/**
	 * Redirect after plugin activation
	 */
	public function redirect_after_activation( $plugin ) {
		// Check if current plugin
		if ( $plugin == PSF_PLUGIN_BASENAME ) {

			// Check if cache build required
			$cache_build_required = (int) get_option( 'psf_cache_build_required' );
			$redirect_url         = 'admin.php?page=product-search-and-filters-settings';

			// Redirect to cache build if it is required
			if ( $cache_build_required ) {
				$redirect_url .= '&cache-wizard';
			}

			// Finally, redirect to settings or cache wizard
			exit( wp_redirect( admin_url( $redirect_url ) ) );
		}
	}

	/**
	 * Build cache if possible
	 */
	public function may_be_cache_rebuild() {
		// Build cache for products search results
		$cache_class = new Psf_Product_Search_Filters_Cache_Rebuild();
		$cache_class->may_be_build_cache();
	}

	/**
	 * Cache rebuild
	 */
	public function cache_rebuild() {
		// Build cache for products search results
		$cache_class = new Psf_Product_Search_Filters_Cache_Rebuild();
		$cache_class->cache_build_steps();
	}

	public function save_settings(){
		if ( wp_verify_nonce( $_POST['psf_product_nonce'], 'psf-product-search-filter' ) ) {
			$fields = filter_input( INPUT_POST, 'submit', FILTER_SANITIZE_STRING );
			if ( ! empty( $fields ) ) {
				if ( isset( $_POST['search_parameter_fields'] ) && ! empty( $_POST['search_parameter_fields'] ) ) {
					array_push( $_POST['search_params'], 'custom_fields' );
				}

				$cache_build_required      = false;
				$old_settings              = get_option( 'psf_search_settings' );
				$check_keys_build_required = [
					'search_params',
					'exclude_products',
					'exclude_categories',
					'search_parameter_fields',
					'out_of_stock',
				];

				// Check cache build required
				foreach ( $check_keys_build_required as $build_required_fields ) {
					if ( ( isset( $old_settings[ $build_required_fields ] ) != isset( $_POST[ $build_required_fields ] ) ) || ( isset( $old_settings[ $build_required_fields ] ) && isset( $_POST[ $build_required_fields ] ) && $old_settings[ $build_required_fields ] != $_POST[ $build_required_fields ] ) ) {
						$this->may_be_cache_rebuild();
						break;
					}
				}

				// Finally save the settings
				update_option( 'psf_search_settings', $_POST );

				echo '<div class="settings-notices notice notice-success"><p><strong>' . esc_html__( 'Settings saved', 'product-search-and-filters' ) . '</strong></p></div>';
			}
		} else{
			echo '<div class="settings-notices notice notice-error"><p><strong>' . esc_html__( 'Nonce could not be validated. Please try again.', 'product-search-and-filters' ) . '</strong></p></div>';
		}
	}

	/**
	 * Handle product update for cache rebuild
	 *
	 * @param $post_id
	 * @param $post
	 * @param $update
	 */
	function build_product_cache( $post_id, $post, $update ) {
		// Only set for post_type = product!
		if ( 'product' !== $post->post_type ) {
			return;
		}

		// Build cache for products search results
		$cache_class = new Psf_Product_Search_Filters_Cache_Rebuild();
		$cache_class->product_cache_build( $post_id, $update );
	}

	/**
	 * Shorten text string helper function
	 *
	 * @param $x
	 * @param $length
	 *
	 * @return mixed|string
	 */
	public function shorten_text( $x, $length ) {
		if ( strlen( $x ) <= $length ) {
			return $x;
		} else {
			return substr( $x, 0, $length ) . '...';
		}
	}

	/**
	 * Render plugin setting page or Cache Wizard
	 */
	public function render_plugin_settings_page() {
		// Show wizard if it is set in URL
		if ( isset( $_GET['cache-wizard'] ) ) {
			include 'partials/cache-wizard.php';
		} else {
			// Otherwise, settings template by default
			$custom_fields = $this->get_product_custom_fields();
			include 'partials/product-search-settings-display.php';
		}
	}

	/**
	 * Get product custom fields
	 *
	 * @return array
	 */
	public function get_product_custom_fields() {
		$custom_fields  = [];
		$default_fields = $this->get_woo_default_custom_fields();
		global $wpdb;

		$custom_fields_result = $wpdb->get_results( $wpdb->prepare( "SELECT DISTINCT $wpdb->postmeta.meta_key FROM $wpdb->posts, $wpdb->postmeta WHERE post_type = %s
                    AND $wpdb->posts.ID = $wpdb->postmeta.post_id", 'product' ), ARRAY_N );

		if ( is_array( $custom_fields_result ) && count( $custom_fields_result ) ) {
			foreach ( $custom_fields_result as $custom_field ) {
				if ( isset( $custom_field[0] ) && ! in_array( $custom_field[0], $default_fields ) && substr( $custom_field[0], 0, 1 ) != '_' ) {
					$custom_fields[] = $custom_field[0];
				}
			}
		}

		return $custom_fields;
	}

	/**
	 * Array list of exluding custom fields for WooCommerce Product
	 *
	 * @return string[]
	 */
	public function get_woo_default_custom_fields() {
		return [
			'_wp_page_template',
			'_wc_review_count',
			'_wc_rating_count',
			'_wc_average_rating',
			'total_sales',
			'_sku',
			'_regular_price',
			'_sale_price',
			'_tax_status',
			'_tax_class',
			'_manage_stock',
			'_backorders',
			'_sold_individually',
			'_virtual',
			'_downloadable',
			'_download_limit',
			'_download_expiry',
			'_stock',
			'_stock_status',
			'_product_attributes',
			'_product_version',
			'_price',
			'_thumbnail_id',
			'_product_image_gallery',
			'_edit_lock',
			'_edit_last',
			'_uncode_specific_main_width',
			'_uncode_specific_menu_remove',
			'_uncode_specific_menu_opaque',
			'_uncode_specific_menu_no_shadow',
			'_uncode_blocks_list',
			'_uncode_header_full_width',
			'_uncode_header_height',
			'_uncode_header_title',
			'_uncode_header_title_custom',
			'_uncode_header_style',
			'_uncode_header_content_width',
			'_uncode_header_custom_width',
			'_uncode_header_align',
			'_uncode_header_position',
			'_uncode_header_title_size',
			'_uncode_header_title_italic',
			'_uncode_header_featured',
			'_uncode_header_background',
			'_uncode_header_parallax',
			'_uncode_header_overlay_color_alpha',
			'_uncode_header_scroll_opacity',
			'_uncode_header_scrolldown',
			'_uncode_specific_layout_width_custom',
			'_uncode_product_media_size',
			'_uncode_specific_navigation_hide',
			'_uncode_specific_copy_hide',
			'_uncode_specific_footer_uparrow_hide',
			'_uncode_fullpage_type',
			'_uncode_fullpage_opacity',
			'_uncode_scroll_dots',
			'_uncode_empty_dots',
			'_uncode_scroll_history',
			'_uncode_scroll_safe_padding',
			'_uncode_scroll_additional_padding',
			'_uncode_fullpage_mobile',
			'_uncode_secondary_thumbnail_id',
		];
	}

}
