<?php

/**
 * This class is responsible to build cache for products search results
 *
 * @since      1.0.0
 * @package    Psf_Product_Search_Filters_Cache_Rebuild
 * @subpackage Psf_Product_Search_Filters/includes
 * @author     KanKode <hello@kankode.com>
 */
class Psf_Product_Search_Filters_Cache_Rebuild {

	/**
	 * Responsible to get the settings
	 *
	 * @var false|mixed|void
	 */
	protected $settings;

	protected $per_step_products;

	public function __construct() {
		$this->settings          = get_option( 'psf_search_settings' );
		$this->per_step_products = 100;
	}

	/**
	 * Build cache for WooCommerce products search
	 *
	 * @param $post_id
	 * @param $update
	 *
	 * @return false|void
	 */
	public function product_cache_build( $post_id, $update ) {
		$db_cache = $this->get_json_from_db();
		$products = $db_cache['products'];

		// Do not build cache for this product if already excluded by any reason
		if ( ! $this->get_products( $post_id ) ) {

			/**
			 * If already not passed the check of eligible products then remove it
			 * from existing products cache
			 */
			if ( isset( $products[ $post_id ] ) ) {
				unset( $products[ $post_id ] );
			}

			return false;
		}

		$products[ $post_id ] = $this->get_single_product_cache( $post_id );

		$json_data = [
			'categories' => $db_cache['categories'],
			'products'   => $products,
			'attributes' => $db_cache['attributes'],
		];

		$this->store_json_data( $json_data );
	}

	/**
	 * Build cache for WooCommerce products search
	 */
	public function cache_build_steps() {
		$page         = (int) get_option( 'psf_cache_build_page' );
		$page         = ( $page === 0 ) ? 1 : $page;
		$status       = 0;
		$cache_data   = [];
		$new_products = $this->build_products( $page );

		// Set category cache for the first time only
		if ( $page === 1 ) {

			// Set build stats parameters
			$total_products = $this->get_total_product_count();
			update_option( 'psf_cache_total_products', $total_products );

			$cache_data['categories'] = $this->build_category_tree();
			$cache_data['attributes'] = $this->build_attribute_date();
			$cache_data['products'] = $new_products;
		} else {
			// For next steps requests
			$db_cache_temp            = get_option( 'psf_cache_temp' );
			$cache_data['categories'] = $db_cache_temp['categories'];
			$cache_data['attributes'] = $db_cache_temp['attributes'];
			$cache_data['products']   = $new_products + $db_cache_temp['products'];
		}

		// At final step, no further steps needed
		if ( ! count( $new_products ) ) {

			$status = 1;
			update_option( 'psf_cache_build_required', '0' );
			update_option( 'psf_cache_build_page', '0' );

			// Store to JSON
			$this->store_json_data( $cache_data );

			$percentage = 100;
		} else {
			// Update page number for next attempt
			update_option( 'psf_cache_build_page', ( $page + 1 ) );

			$total_products = get_option( 'psf_cache_total_products' );

			// Percentage of completion
			$percentage = $page * $this->per_step_products / $total_products * 100;
			$percentage = ( $percentage > 100 ) ? 100 : $percentage;

			// Update cache into db temporary
			update_option( 'psf_cache_temp', $cache_data );
		}

		$data = [
			'page_number' => $page,
			'percentage'  => $percentage,
			'status'      => $status,
		];

		$this->ajax_response( $data );
	}

	/**
	 * Build cache for WooCommerce products search
	 */
	public function cache_build() {
		$product_count = (int) $this->get_total_product_count();

		// If product count is more than 1000 then require for cache build wizard
		if ( $product_count > PSF_MAX_PRODUCT_COUNT ) {
			update_option( 'psf_cache_build_required', 1 );

			return false;
		}

		$json_data = [
			'categories' => $this->build_category_tree(),
			'products'   => $this->build_products(),
			'attributes' => $this->build_attribute_date(),
		];

		$this->store_json_data( $json_data );
	}

	/**
	 * Build cache if possible
	 */
	public function may_be_build_cache(){
		global $wpdb;

		// Get total product count
		$product_count = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM $wpdb->posts WHERE post_type = '%s' AND post_status = '%s'", 'product', 'publish' ) );

		// If product count is less than or equal to 1000 then build product cache right away.
		if ( $product_count <= PSF_MAX_PRODUCT_COUNT ) {
			// Build cache for products search results
			$cache_class = new Psf_Product_Search_Filters_Cache_Rebuild();
			$cache_class->cache_build();

			// Now cache building is not required as it is done on activation
			update_option( 'psf_cache_build_required', '0' );
		} else {
			update_option( 'psf_cache_build_required', '1' );
		}
	}

	/**
	 * Build cache data for products
	 *
	 * @return array
	 */
	public function build_products( $page = 0 ) {

		$products      = $this->get_products( 0, $page );
		$products_json = [];

		if ( is_array( $products ) && count( $products ) ) {
			foreach ( $products as $value ) {
				$get_product_cache = $this->get_single_product_cache( $value->ID );

				if ( count( $get_product_cache ) ) {
					$products_json[ $value->ID ] = $get_product_cache;
				}
			}
		}

		return $products_json;
	}

	/**
	 * Get products based on settings and criteria
	 *
	 * @param int $product_id
	 * @param int $page
	 *
	 * @return int[]|\WP_Post[]
	 */
	public function get_products( $product_id = 0, $page = 0 ) {
		// Prepare query arguments
		$page                   = (int) $page;
		$args                   = [];
		$args['post_type']      = 'product';
		$args['posts_per_page'] = ( isset( $this->settings['cache_size'] ) ) ? $this->settings['cache_size'] : '-1';

		if ( isset( $this->settings['exclude_products'] ) ) {
			$args['exclude'] = $this->settings['exclude_products'];
		}

		if ( isset( $this->settings['exclude_categories'] ) ) {
			$args['tax_query'][] = [
				'taxonomy' => 'product_cat',
				'field'    => 'term_id',
				'terms'    => $this->settings['exclude_categories'],
				'operator' => 'NOT IN',
			];
		}

		if ( $this->settings['out_of_stock'] == 'yes' ) {
			$args['meta_query'][] = [
				'key'     => '_stock_status',
				'value'   => 'outofstock',
				'compare' => '!=',
			];
		}

		// Check for specific post ID
		if ( $product_id > 0 ) {
			$args['post__in'] = [ $product_id ];
		}

		// Handle steps
		if ( $page > 0 ) {

			// Handle offset
			$offset = ( $page === 1 ) ? 0 : ( $page - 1 ) * $this->per_step_products;

			$args['posts_per_page'] = $this->per_step_products;
			$args['offset']         = $offset;
		}

		// Get the products
		$products = get_posts( $args );

		return ( is_array( $products ) && count( $products ) ) ? $products : false;
	}

	/**
	 * Get cache for a single product by ID
	 *
	 * @param $product_id
	 *
	 * @return array|false
	 */
	public function get_single_product_cache( $product_id ) {

		$product_woo = wc_get_product( $product_id );
		$attributes  = [];
		$product     = false;

		if ( $product_woo->get_type() != 'grouped' ) {
			$stock_status = $product_woo->get_stock_status();

			if ( $stock_status == 'instock' ) {
				$stock_status = '<span class="stock_status ' . esc_attr( $stock_status ) . '"><svg xmlns="http://www.w3.org/2000/svg" x="0px" y="0px" width="24" height="24" viewBox="0 0 172 172" style=" fill:#000000;"><g fill="none" fill-rule="nonzero" stroke="none" stroke-width="1" stroke-linecap="butt" stroke-linejoin="miter" stroke-miterlimit="10" stroke-dasharray="" stroke-dashoffset="0" font-family="none" font-weight="none" font-size="none" text-anchor="none" style="mix-blend-mode: normal"><path d="M0,172v-172h172v172z" fill="none"></path><g fill="#000000"><path d="M145.43294,37.93294l-80.93294,80.93295l-30.76628,-30.76628l-10.13411,10.13411l40.90039,40.90039l91.06706,-91.06705z"></path></g></g></svg> In Stock</span>';
			}

			if ( $stock_status == 'outofstock' ) {
				$stock_status = '<span class="stock_status ' . esc_attr( $stock_status ) . '"><svg width="133" height="133" viewBox="0 0 133 133" fill="none" xmlns="http://www.w3.org/2000/svg">
<path d="M5.98848 132.018C4.80547 132.013 3.65032 131.659 2.66825 130.999C1.68618 130.34 0.921032 129.404 0.468982 128.311C0.0169324 127.218 -0.101843 126.015 0.127594 124.855C0.357031 123.694 0.92444 122.627 1.75849 121.788L121.758 1.78837C122.316 1.23091 122.978 0.788705 123.706 0.487009C124.434 0.185313 125.215 0.0300293 126.003 0.0300293C126.792 0.0300293 127.572 0.185313 128.301 0.487009C129.029 0.788705 129.691 1.23091 130.248 1.78837C130.806 2.34583 131.248 3.00763 131.55 3.73599C131.852 4.46435 132.007 5.245 132.007 6.03336C132.007 6.82173 131.852 7.60238 131.55 8.33074C131.248 9.0591 130.806 9.72091 130.248 10.2784L10.2485 130.278C9.68784 130.834 9.02295 131.274 8.29192 131.573C7.5609 131.872 6.77812 132.023 5.98848 132.018Z" fill="black"/>
<path d="M125.988 132.018C124.4 132.012 122.878 131.375 121.758 130.248L1.75834 10.2483C1.20088 9.69088 0.758676 9.02907 0.45698 8.30071C0.155284 7.57236 1.55406e-08 6.79171 0 6.00334C-1.55406e-08 5.21497 0.155284 4.43432 0.45698 3.70596C0.758676 2.9776 1.20088 2.3158 1.75834 1.75834C2.3158 1.20088 2.9776 0.758676 3.70596 0.45698C4.43432 0.155284 5.21497 -1.55406e-08 6.00334 0C6.7917 1.55406e-08 7.57236 0.155284 8.30071 0.45698C9.02907 0.758676 9.69088 1.20088 10.2483 1.75834L130.248 121.758C131.094 122.598 131.672 123.67 131.906 124.838C132.14 126.007 132.022 127.219 131.565 128.319C131.108 129.42 130.333 130.36 129.34 131.018C128.347 131.677 127.18 132.025 125.988 132.018Z" fill="black"/>
</svg> Out of Stock</span>';
			}

			if ( $stock_status == 'onbackorder' ) {
				$stock_status = '<span class="stock_status ' . esc_attr( $stock_status ) . '"><svg xmlns="http://www.w3.org/2000/svg" x="0px" y="0px" width="24" height="24" viewBox="0 0 172 172" style=" fill:#000000;"><g fill="none" fill-rule="nonzero" stroke="none" stroke-width="1" stroke-linecap="butt" stroke-linejoin="miter" stroke-miterlimit="10" stroke-dasharray="" stroke-dashoffset="0" font-family="none" font-weight="none" font-size="none" text-anchor="none" style="mix-blend-mode: normal"><path d="M0,172v-172h172v172z" fill="none"></path><g fill="#000000"><path d="M145.43294,37.93294l-80.93294,80.93295l-30.76628,-30.76628l-10.13411,10.13411l40.90039,40.90039l91.06706,-91.06705z"></path></g></g></svg> On backorder</span>';
			}

			// Get product thumbnail
			$thumbnail = get_the_post_thumbnail_url( $product_id );

			$product                            = [];
			$product['id']                      = $product_id;
			$product['type']                    = $product_woo->get_type();
			$product['name']                    = get_the_title( $product_id );
			$product['permalink']               = get_permalink( $product_id );
			$product['short_description']       = $product_woo->get_short_description();
			$product['long_description']        = $product_woo->get_description();
			$product['sku']                     = $product_woo->get_sku();
			$product['price']                   = $product_woo->get_price();
			$product['stock_status']            = $stock_status;
			$product['price_symbol']            = get_woocommerce_currency_symbol();
			$product['tax_status']              = $product_woo->get_tax_status();
			$product['regular_price']           = $product_woo->get_regular_price();
			$product['sale_price']              = $product_woo->get_sale_price();
			$product['on_sale']                 = $product_woo->is_on_sale();
			$product['image']                   = ( ! empty( $thumbnail ) ) ? $thumbnail : wc_placeholder_img_src();
			$product['brand']                   = $product_woo->get_attribute( 'pa_brand' ); // by default attribute support for WooCommerce Brands plugin
			$product['brand_image']             = '';
			$product['search_parameter_fields'] = '';

			// Handle multiple custom fields
			if ( isset( $this->settings['search_parameter_fields'] ) && is_array( $this->settings['search_parameter_fields'] ) && count( $this->settings['search_parameter_fields'] ) ) {
				foreach ( $this->settings['search_parameter_fields'] as $custom_field ) {
					$meta_value = (string) get_post_meta( $product_id, $custom_field, true );
					if ( ! empty( $meta_value ) ) {
						$product['search_parameter_fields'] .= $meta_value;
					}
				}
			}

			// Attach branch image
			if ( $product['brand'] ) {
				$brand                  = get_term_by( 'slug', strtolower( $product['brand'] ), 'product_brand' );
				$product['brand_image'] = wp_get_attachment_image_url( get_term_meta( $brand->term_id, 'thumbnail_id', true ), 'full' );
			}

			if ( empty( $product['brand_image'] ) && $this->settings['display_product_brand_logo'] == 'yes' ) {
				$brand = get_the_terms( $product['id'], 'product_brand' );

				if ( is_array( $brand ) && isset( $brand[0] ) && gettype( $brand[0] ) === 'object' && isset( $brand[0]->term_id ) ) {
					$product['brand_image'] = wp_get_attachment_image_url( get_term_meta( $brand[0]->term_id, 'thumbnail_id', true ), 'full' );
				}
			}

			if ( isset( $this->settings['sidebar_attributes'] ) && is_array( $this->settings['sidebar_attributes'] ) && count( $this->settings['sidebar_attributes'] ) ) {
				foreach ( $this->settings['sidebar_attributes'] as $attr ) {
					$attr_name = str_replace( '-', ' ', str_replace( 'pa_', '', $attr ) );
					$attr_name = ucwords( $attr_name );
					if ( $product_woo->get_attribute( $attr ) ) {
						$attributes[ $attr_name ] = explode( ', ', $product_woo->get_attribute( $attr ) );
					} else {
						$attributes[ $attr_name ] = [];
					}
				}
			}

			$cats       = get_the_terms( $product_id, 'product_cat' );
			$categories = [];
			if ( is_array( $cats ) && count( $cats ) ) {
				foreach ( $cats as $key => $s_cat ) {
					$cat         = [];
					$cat['id']   = $s_cat->term_id;
					$cat['name'] = $s_cat->name;
					$cat['slug'] = $s_cat->slug;

					$categories[] = $cat;
				}
			}

			$product['cat']        = $categories;
			$product['attributes'] = $attributes;
		}

		return $product;
	}

	/**
	 * Build cache data for attributes
	 *
	 * @return array
	 */
	public function build_attribute_date() {
		$attributes_tree = [];

		if ( isset( $this->settings['sidebar_attributes'] ) && is_array( $this->settings['sidebar_attributes'] ) && count( $this->settings['sidebar_attributes'] ) ) {
			foreach ( $this->settings['sidebar_attributes'] as $attr ) {
				$attr_name                     = str_replace( '-', ' ', str_replace( 'pa_', '', $attr ) );
				$attr_name                     = ucwords( $attr_name );
				$attributes_tree[ $attr_name ] = [];
				foreach ( get_terms( $attr ) as $term ) {
					$attributes_tree[ $attr_name ][] = [
						'id'   => $term->term_id,
						'name' => $term->name,
						'slug' => $term->slug,
					];
				}
			}
		}

		return $attributes_tree;
	}

	/**
	 * Build cache data for categories tree
	 *
	 * @return array
	 */
	public function build_category_tree() {
		$categories_tree    = [];
		$exclude_categories = isset( $this->settings['exclude_categories'] ) ? $this->settings['exclude_categories'] : '';
		$categories         = get_terms( 'product_cat', [
			'parent'  => 0,
			'exclude' => $exclude_categories,
		] );

		if ( is_array( $categories ) && count( $categories ) ) {
			foreach ( $categories as $s_cat ) {
				$cat           = [];
				$cat['id']     = $s_cat->term_id;
				$cat['name']   = $s_cat->name;
				$cat['slug']   = $s_cat->slug;
				$cat['childs'] = [];

				$childs = get_terms( 'product_cat', [
					'parent'  => $s_cat->term_id,
					'exclude' => $exclude_categories,
				] );
				if ( $childs ) {
					$all_childs = [];
					if ( is_array( $childs ) && count( $childs ) ) {
						foreach ( $childs as $child ) {
							$chi         = [];
							$chi['id']   = $child->term_id;
							$chi['name'] = $child->name;
							$chi['slug'] = $child->slug;
							array_push( $all_childs, $chi );
						}
					}
					$cat['childs'] = $all_childs;
				}

				array_push( $categories_tree, $cat );
			}
		}

		return $categories_tree;
	}

	/**
	 * Store JSON data into the file
	 *
	 * @param $json_data
	 */
	public function store_json_data( $json_data ) {
		$this->store_json_to_db( $json_data );
		$json_data['products'] = array_values( $json_data['products'] );

		WP_Filesystem();
		global $wp_filesystem;

		$upload_dir      = wp_upload_dir();
		$upload_base_dir = $upload_dir['basedir'];
		$data_json_dir   = $upload_base_dir . '/product-search-and-filters';

		// Make product-search-and-filters directory in uploads if not exists already.
		if ( ! $wp_filesystem->is_dir( $data_json_dir ) ) {
			$wp_filesystem->mkdir( $data_json_dir, 0755 );
		}

		$json_file_path = $data_json_dir . '/data.json';

		// Write file with WooCommerce products search results
		$wp_filesystem->put_contents( $json_file_path, json_encode( $json_data, JSON_UNESCAPED_SLASHES ) );
	}

	/**
	 * Store JSON data to database
	 *
	 * @param $json_data
	 */
	public function store_json_to_db( $json_data ) {
		update_option( 'psf_cache', $json_data );
	}

	/**
	 * Get JSON data from database
	 *
	 * @return false|mixed|void
	 */
	public function get_json_from_db() {
		return get_option( 'psf_cache' );
	}

	/**
	 * Get total product count
	 *
	 * @return string|null
	 */
	public function get_total_product_count() {
		global $wpdb;

		// Get total product count
		return $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM $wpdb->posts WHERE post_type = '%s' AND post_status = '%s'", 'product', 'publish' ) );
	}

	/**
	 * Send ajax response
	 */
	private function ajax_response( $data ) {
		header( 'Content-Type: application/json' );

		echo wp_json_encode( $data );
		wp_die();
	}
}
