<?php

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Product_Search_Filters
 * @subpackage Psf_Product_Search_Filters/public
 * @author     KanKode <hello@kankode.com>
 */
class Psf_Product_Search_Filters_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of the plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version     = $version;
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		// Enqueue styles
		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/styles.css', [], $this->version, 'all' );
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		// Use WordPress vendor libs for jquery, underscore and lodash along with custom scripts
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/scripts.js', [
			'jquery',
			'underscore',
			'lodash',
		], $this->version, true );

		$upload        = wp_upload_dir();
		$upload_dir    = $upload['baseurl'];
		$data_json_dir = $upload_dir . '/product-search-and-filters';

		wp_localize_script( $this->plugin_name, 'psfSearchConfigs', [
			'settings'         => get_option( 'psf_search_settings' ),
			'url'              => $data_json_dir . '/data.json',
			'noResultFoundTxt' => esc_html__( 'No results found matching', 'product-search-and-filters' ),
			'addToCartTxt'     => esc_html__( 'Add to cart', 'product-search-and-filters' ),
			'selectOptionsTxt'     => esc_html__( 'Select options', 'product-search-and-filters' ),
			'viewProductTxt'   => esc_html__( 'View product', 'product-search-and-filters' ),
			'viewCartTxt'      => esc_html__( 'View cart', 'product-search-and-filters' ),
			'cartLink'         => esc_url( wc_get_cart_url() ),
			'ajaxUrl'          => esc_url( admin_url( 'admin-ajax.php' ) ),
		] );
	}

	/**
	 * Add to cart a product using Ajax from front popup
	 */
	public function add_to_cart() {
		$data = [
			'msg'    => esc_attr__( 'Could not add to cart!', 'product-search-and-filters' ),
			'status' => 0,
		];

		$product_id = isset( $_POST['product_id'] ) ? (int) $_POST['product_id'] : 0;

		// Check if valid post ID number
		if ( $product_id <= 0 ) {
			$data['msg'] = esc_attr__( 'Invalid product ID', 'product-search-and-filters' );
			$this->ajax_response( $data );
		}

		// Check if post_type is product
		if ( 'product' !== get_post_type( $product_id ) ) {
			$data['msg'] = esc_attr__( 'This is not a valid product ID', 'product-search-and-filters' );
			$this->ajax_response( $data );
		}

		// Finally, add to cart
		if ( function_exists( 'WC' ) ) {
			try {
				$add_to_cart = WC()->cart->add_to_cart( $product_id );

				if ( $add_to_cart ) {
					$data['msg']    = esc_attr__( 'Added to cart!', 'product-search-and-filters' );
					$data['status'] = 1;
				}
			} catch ( Exception $e ) {
				$data['msg'] = $e->getMessage();
			}
		}

		$this->ajax_response( $data );
	}

	public function render_plugin_popup_section() {
		include 'partials/product-search-popup.php';
	}

	public function render_plugin_search_section() {
		$settings = get_option( 'psf_search_settings' );
		echo '<input type="text" id="smart_search" name="smart_search" placeholder="' . esc_attr( $settings['search_placeholder'] ) . '">';
	}

	/**
	 * Send ajax response
	 */
	private function ajax_response( $data ) {
		header( 'Content-Type: application/json' );

		echo wp_json_encode( $data );
		wp_die();
	}
}
